/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef BEAT_H_
#define BEAT_H_

#include <iostream>
#include <thread>
#include <mutex>
#include <condition_variable>
#include <vector>

class Beat {
    std::mutex beat_guard_;
    std::condition_variable state_update_;
    std::condition_variable beat_define_;
public:
    Beat(size_t beat_flags_number,double beat_time,double beat_interval,double music_reliability,int sampling_rate)
        :beat_flags_(beat_flags_number,false) {
        beat_time_ = beat_time;
        beat_interval_ = beat_interval;
        music_reliability_ = music_reliability;
        sampling_rate_ = sampling_rate;
        on_music_flag_ = false;
        beat_define_flag_ = false;
        music_number_ = 0;
        beat_number_ = 0;
    };
    ~Beat() {};
    double beat_time() {
        std::lock_guard<std::mutex> lk(beat_guard_);
        return beat_time_;
    };
    int time_by_msec() {
        std::lock_guard<std::mutex> lk(beat_guard_);
        return beat_time_*1000/sampling_rate_;
    }
    double beat_interval() {
        std::lock_guard<std::mutex> lk(beat_guard_);
        return beat_interval_;
    };
    int interval_by_msec() {
        std::lock_guard<std::mutex> lk(beat_guard_);
        return beat_interval_*1000/sampling_rate_;
    };
    double music_reliability() {
        std::lock_guard<std::mutex> lk(beat_guard_);
        return music_reliability_;
    };
    bool on_music_flag() {
        std::lock_guard<std::mutex> lk(beat_guard_);
        return on_music_flag_;
    }
    void set_on_music_flag(bool on_music_flag) {
        std::lock_guard<std::mutex> lk(beat_guard_);
        on_music_flag_ = on_music_flag;
    }
    void set_beat(double beat_time,double beat_interval,double music_reliability) {
        std::lock_guard<std::mutex> lk(beat_guard_);
        beat_time_=beat_time;
        beat_interval_=beat_interval;
        music_reliability_=music_reliability;
        state_update_.notify_all();
    };
    void set_music_reliability(double music_reliability) {
        std::lock_guard<std::mutex> lk(beat_guard_);
        music_reliability_=music_reliability;
    };

    void WaitForUpdate() {
        std::unique_lock<std::mutex> lk(beat_guard_);
        state_update_.wait(lk);
    };
    void BeatDefine() {
        std::lock_guard<std::mutex> lk(beat_guard_);
        beat_define_flag_ = false;
        beat_define_.notify_all();
    }
    void WaitForBeatDefine() {
        std::unique_lock<std::mutex> lk(beat_guard_);
        beat_define_.wait(lk);
    }
    bool beat_flags_by_num(size_t flag_number) {
        std::lock_guard<std::mutex> lk(beat_guard_);
        if(flag_number < beat_flags_.size()) {
            return beat_flags_[flag_number];
        }
        return false;
    }
    bool beat_define_flag() {
        std::lock_guard<std::mutex> lk(beat_guard_);
        return beat_define_flag_;
    }
    size_t music_number() {
        std::lock_guard<std::mutex> lk(beat_guard_);
        return music_number_;
    }
    int beat_number() {
        std::lock_guard<std::mutex> lk(beat_guard_);
        return beat_number_;
    }
    int set_beat_flags(size_t flag_number,bool value) {
        std::lock_guard<std::mutex> lk(beat_guard_);
        if(flag_number < beat_flags_.size()) {
            beat_flags_[flag_number]=value;
            return 0;
        }
        return -1;
    };
    int set_beat_define_flag(bool value) {
        std::lock_guard<std::mutex> lk(beat_guard_);
        beat_define_flag_ = value;
        return 0;
    }
    int set_beat_number(int beat_number) {
        std::lock_guard<std::mutex> lk(beat_guard_);
        beat_number_=beat_number;
        return 0;
    }
    int set_music_number(size_t music_number) {
        std::lock_guard<std::mutex> lk(beat_guard_);
        music_number_ = music_number;
        return 0;
    }
    void reset_all_beat_flags(bool value) {
        std::lock_guard<std::mutex> lk(beat_guard_);
        beat_flags_.assign(beat_flags_.size(),value);
        beat_define_flag_ = value;
    }
protected:
    double beat_time_;
    double beat_interval_;
    double music_reliability_;
    int sampling_rate_;
    std::vector<bool> beat_flags_;
    bool beat_define_flag_;
    bool on_music_flag_;
    size_t music_number_;
    int beat_number_;
};



#endif /* BEAT_H_ */
