/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef ___FFTW_HPP___
#define ___FFTW_HPP___

#if 0

#include <complex>
#include <vector>


#include <boost/shared_array.hpp>
#include <boost/type_traits.hpp>
#include <boost/numeric/ublas/vector.hpp>

#include <fftw3.h>

template <typename T>
struct plan_selector {};

template <>
struct plan_selector<float> {
  typedef fftwf_plan result;
};

template <>
struct plan_selector<double> {
  typedef fftw_plan result;
};


template <typename T>
class FFT {
public:
  typedef T value_type;
  typedef typename plan_selector<T>::result fft_plan;
    
private:
  fft_plan plan_forward;
  fft_plan plan_backward;

public:
  size_t length;                    // length of signal
  size_t slength;                   // length of spectrum
  T* real_data;
  std::complex<T>* complex_data;

public:
  FFT() {}
  FFT(size_t n) {
    Initialize(n);
  }

  ~FFT() {
    // I had better free real_data, complex_data, and plan_*.
    DestroyPlan();
    fftwf_free(real_data);
    fftwf_free(complex_data);
  }

  void Initialize(size_t n) {
    length = n;
    slength = n / 2 + 1;
        
    real_data = (T*)fftwf_malloc(sizeof(T) * length);
    complex_data = (std::complex<T>*)fftwf_malloc(sizeof(std::complex<T>) *
                                                  slength);

    InitializePlan();
  }

  void CalcForward() {
    fftwf_execute(plan_forward);
  }

  void CalcBackward() {
    fftwf_execute(plan_backward);
  }

private:
  void InitializePlan();
  void DestroyPlan();
};


class DCT {
private:
  fftwf_plan plan_forward;
  fftwf_plan plan_backward;

public:
  size_t length;                    // length of signal
  float* in;
  float* out;

public:
  DCT() {}
    
  DCT(size_t n)
  {
    Initialize(n);
  }

  ~DCT()
  {
    // I had better free real_data, complex_data, and plan_*.
    fftwf_destroy_plan(plan_forward);
    fftwf_destroy_plan(plan_backward);
    fftwf_free(in);
    fftwf_free(out);
  }

  void Initialize(size_t n)
  {
    length = n;

    in = (float*)fftwf_malloc(sizeof(float) * length);
    out = (float*)fftwf_malloc(sizeof(float) * length);

    plan_forward = fftwf_plan_r2r_1d(length, in, out, FFTW_REDFT10, FFTW_ESTIMATE);
    plan_backward = fftwf_plan_r2r_1d(length, out, in, FFTW_REDFT01, FFTW_ESTIMATE);
  }

  void CalcForward() {
    fftwf_execute(plan_forward);
    /*
      out[0] /= 2 * sqrtf(length);
      for (int i = 1; i < length; i++) {
      out[i] /= sqrtf(2 * length);
      }
    */
  }

  void CalcBackward() {
    fftwf_execute(plan_backward);
    for (int i = 0; i < length; i++) {
      //in[i] = (sqrtf(2) - 1) / sqrtf(2 * length) * out[0] + in[i] / sqrtf(2 * length);
      in[i] /= 2.0f * length;
    }
  }

};

template <typename T>
void Hanning(boost::numeric::ublas::vector<T>& w)
{
  float c = 2.0 * M_PI / (w.size() - 1);
  for (int n = 0; n < w.size(); n++) {
    w[n] = 0.5 - 0.5 * cos(c * n);
  }
}

template <typename T>
void Hamming(boost::numeric::ublas::vector<T>& w)
{
  float c = 2.0 * M_PI / (w.size() - 1);
  for (int n = 0; n < w.size(); n++) {
    w[n] = 0.54 - 0.46 * cos(c * n);
  }
}

template <typename T>
void ConjWindow(boost::numeric::ublas::vector<T>& w)
{
  for (int i = 0; i < w.size(); i++) {
    T x = 4 * i / (T)w.size();
    int inv = 0;

    if (x < 1) {
    }
    else if (x < 2) {
      x = 2 - x;
      inv = 1;
    }
    else if (x < 3) {
      x = x - 2;
      inv = 1;
    }
    else {
      x = 4 - x;
    }
    x *= 1.9979;
    w[i] = (0.5 - 0.5 * cos(x)) * (0.5 - 0.5 * cos(x));
    if (inv) {
      w[i] = 1 - w[i];
    }
    w[i] = sqrt(w[i]);
  }
}

#endif

#endif
