/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include <vector>
#include <iostream>
#include <complex>
#include <cmath>
#include <limits>
#include <beattracker/fft_analysis.h>
#include <beattracker/window_function.h>

using namespace std;


double convert_log_scale(double x, double pow) {
    if (x <= 0)
        return 0;

    const double value = log10(x) + pow;
    return (value > 0) ? value : 0;
}

FftComplex::FftComplex(double real, double imag) {
    data_[0] = real;
    data_[1] = imag;
}

double FftComplex::abs() const {
    return sqrt(data_[0] * data_[0] + data_[1] * data_[1]);
}

#if EXTERNAL_FFT == 0
FftAnalysis::FftAnalysis(size_t win_size) :
    window_(win_size) {
    win_size_ = win_size;
    initialize();
}

FftAnalysis::FftAnalysis(const WindowFunction& window) :
    window_(window) {
    win_size_ = window_.size();
    initialize();
}

void FftAnalysis::initialize() {
    // allocate memory
    r_data_ = (double*) fftw_malloc(sizeof(double) * win_size_);
    r_out_data_ = (double*) fftw_malloc(sizeof(double) * win_size_);
    c_data_ = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * (win_size_ / 2
                                          + 1));

    // create fftw-plan

    p_r2c_ = fftw_plan_dft_r2c_1d(win_size_, r_data_, c_data_, FFTW_ESTIMATE);
    p_c2r_ = fftw_plan_dft_c2r_1d(win_size_, c_data_, r_data_, FFTW_ESTIMATE);
    p_r2r_= fftw_plan_r2r_1d(win_size_, r_data_, r_out_data_, FFTW_REDFT10, FFTW_ESTIMATE);

    /*
      p_r2c_ = fftw_plan_dft_r2c_1d(win_size_, r_data_, c_data_, FFTW_PATIENT);
      p_c2r_ = fftw_plan_dft_c2r_1d(win_size_, c_data_, r_data_, FFTW_PATIENT);
      p_r2r_= fftw_plan_r2r_1d(win_size_, r_data_, r_out_data_, FFTW_REDFT10, FFTW_ESTIMATE);
      */

}

FftAnalysis::~FftAnalysis() throw() {
    // free memory
    fftw_free(r_data_);
    fftw_free(r_out_data_);
    fftw_free(c_data_);
}

fftw_complex* FftAnalysis::alloc_fft_location(size_t size) const {
    fftw_complex *ptr;

#ifdef USE_FFTW_MALLOC
    ptr = (fftw_complex *)fftw_malloc(sizeof(fftw_complex) * size);
#else
    ptr = new fftw_complex[size];
#endif

    return ptr;
}

void FftAnalysis::free_fft_location(fftw_complex* ptr) const {
#ifdef USE_FFTW_MALLOC
    free_fftw_location(ptr);
#else
    delete[] ptr;
#endif
}

void FftAnalysis::execute_fourier_transform(
    vector<double>::const_iterator first, vector<FftComplex>& spectrum) {

    // multiply window function
    for (size_t i = 0; i < win_size_; i++) {
        //    in[i][0] = (*first) * window_[i];
        //    in[i][1] = 0.0;
        r_data_[i] = (*first) * window_[i];
        ++first;
    }

    // execute FFT
    //fftw_execute(plan);
    fftw_execute(p_r2c_);

    // save result
    spectrum.resize(win_size_ / 2 + 1);

    for (size_t i = 0; i < (win_size_ / 2 + 1); i++) {
        //    spectrum[i][0] = out[i][0];
        //    spectrum[i][1] = out[i][1];
        spectrum[i][0] = c_data_[i][0];
        spectrum[i][1] = c_data_[i][1];
    }
}

void FftAnalysis::execute_fourier_transform(const vector<double>& signal,
        vector<FftComplex>& spectrum) {
    execute_fourier_transform(signal.begin(), spectrum);
}

void FftAnalysis::execute_DCTransform(vector<double>::const_iterator first,
                                      vector<double>& spectrum) {

    // multiply window function
    for (size_t i = 0; i < win_size_; i++) {
        r_data_[i] = *first;
        ++first;
    }

    // execute FFT
    fftw_execute(p_r2r_);

    // save result
    for (size_t i = 0; i < win_size_; i++) {
        spectrum[i] = r_out_data_[i];
    }
}

void FftAnalysis::execute_DCTransform(const vector<double>& signal, vector<
                                      double>& spectrum) {
    execute_DCTransform(signal.begin(), spectrum);
}

void FftAnalysis::execute_inverse_transform(const vector<FftComplex>& spectrum,
        vector<double>& signal, bool windowed) {
    // set input array
    for (size_t i = 0; i < (win_size_ / 2 + 1); i++) {
        //    in[i][0] = spectrum[i][0];
        //    in[i][1] = spectrum[i][1];
        c_data_[i][0] = spectrum[i][0];
        c_data_[i][1] = spectrum[i][1];
    }

    // execute IFFT
    //fftw_execute(plan);
    fftw_execute(p_c2r_);

    // save result
    signal.resize(win_size_);
    for (size_t i = 0; i < win_size_; i++)
        signal[i] = r_data_[i];

    if (windowed)
        for (size_t i = 0; i < win_size_; i++)
            signal[i] *= window_[i];
}
#endif

