/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include <vector>
#include <cmath>
#include <beattracker/window_function.h>

#include <iostream>

using namespace std;

const double WindowFunction::pi = 3.1415926;

WindowFunction::WindowFunction(size_t win_size, Type win_type)
{
    data_.assign(win_size, 0.0);
    win_type_ = win_type;

    vector<double> x(win_size);

    for(size_t i = 0; i < win_size; i++)
        x[i] = static_cast<double>(i) / win_size;

    switch(win_type) {
    case WIN_HANNING:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 0.50
                       - (0.50 * cos(2.0 * pi * x[i]));
        break;

    case WIN_HAMMING:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 0.54
                       - (0.46 * cos(2.0 * pi * x[i]));
        /*
          {
        float c = 2.0 * M_PI / (win_size - 1);
        for (int n = 0; n < win_size; n++) {
          data_[n] = 0.54 - 0.46 * cos(c * n);
        }
          }
          */
        break;

    case WIN_BARTLETT:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 1.0
                       - (2.0 * abs(x[i] - 0.5));
        break;

    case WIN_BARTLETT_HANN:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 0.62
                       - (0.48 * abs(x[i] - 0.5))
                       - (0.38 * cos(2.0 * pi * x[i]));
        break;

    case WIN_NUTTALL:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 0.355768
                       - (0.487396 * cos(2.0 * pi * x[i]))
                       + (0.144232 * cos(4.0 * pi * x[i]))
                       - (0.012604 * cos(6.0 * pi * x[i]));
        break;

    case WIN_BLACKMAN:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 0.42
                       - (0.50 * cos(2.0 * pi * x[i]))
                       + (0.08 * cos(4.0 * pi * x[i]));
        break;

    case WIN_BLACKMAN_HARRIS:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 0.35875
                       - (0.48829 * cos(2.0 * pi * x[i]))
                       + (0.14128 * cos(4.0 * pi * x[i]))
                       - (0.01168 * cos(6.0 * pi * x[i]));
        break;

    case WIN_BLACKMAN_NUTTALL:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 0.3635819
                       - (0.4891775 * cos(2.0 * pi * x[i]))
                       + (0.1365995 * cos(4.0 * pi * x[i]))
                       - (0.0106411 * cos(6.0 * pi * x[i]));
        break;

    case WIN_FLATTOP:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 1.0
                       - (1.93  * cos(2.0 * pi * x[i]))
                       + (1.29  * cos(4.0 * pi * x[i]))
                       + (0.388 * cos(6.0 * pi * x[i]))
                       + (0.032 * cos(8.0 * pi * x[i]));
        break;

    case WIN_AKAIKE:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 0.625
                       - (0.500 * cos(2.0 * pi * x[i]))
                       - (0.125 * cos(4.0 * pi * x[i]));
        break;

    case WIN_WELCH:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 4.0 * x[i] * (1.0 - x[i]);
        break;

    case WIN_ROSENFIELD:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 0.762
                       - (1.000 * cos(2.0 * pi * x[i]))
                       + (0.238 * cos(4.0 * pi * x[i]));
        break;

    default:
        for(size_t i = 0; i < win_size; i++)
            data_[i] = 1.0;
    }
}

WindowFunction::~WindowFunction()throw()
{
}

size_t WindowFunction::size()const
{
    return data_.size();
}

WindowFunction::Type WindowFunction::type()const
{
    return win_type_;
}

double WindowFunction::operator[](size_t i)const
{
    return data_[i];
}

double WindowFunction::value(size_t i)const
{
    return data_[i];
}
